#pragma once

#include <iostream>
#include <vector>
#include <fstream>
#include <algorithm>
#include <string>
#include <sstream>
#include <iterator>


void createData_filestream()
{
	std::ofstream out("numbers.txt");
	for (int i = 0; i < 10; ++i)
		out << rand() % 51 - 25 << " ";
}

void createData_generate()
	{
	std::vector<int> v(10);
	std::generate(v.begin(), v.end(), []() {return rand() % 51 - 25; });
	std::ofstream out("numbers.txt");
	std::copy(v.begin(), v.end(), std::ostream_iterator<int>(out, " "));
}


template <typename _Container>
_Container loadData_filestream(const std::string& filename)
{
	_Container m_container;

	std::ifstream in(filename);
	if (!in)
	{
		std::cerr << "File cannot be opened" << std::endl;
		exit;
	}

	std::string line;
	while (std::getline(in, line, ' '))
	{
		std::istringstream iss(line);
		typename _Container::value_type elem;
		if (!(iss >> elem))
			break;
		m_container.push_back(elem);
	}
	return m_container;
}

template<typename _Container>
_Container loadData_copy(const std::string& filename)
{
	_Container m_container(10);
	std::ifstream in(filename);
	if (!in) {
		std::cerr << "File cannot be opened" << std::endl;
		exit;
	}
	std::copy(std::istream_iterator<typename _Container::value_type>(in), std::istream_iterator<typename _Container::value_type>(), m_container.begin());
	return m_container;
}

template<typename _Container>	
_Container modify_template(const _Container& mycont)
{
	_Container res;
	typename _Container::value_type sum = 0;
	for (auto& elem : mycont)
		sum += elem;
	
	for (auto& elem : mycont)
	{
		if (elem == 0)
			res.push_back((double)sum / mycont.size());
		else
			res.push_back(elem);
	}
	std::cout << "\n\n\n";
	for (const auto& elem : mycont)
		std::cout << elem << " ";
	std::cout << "\n\n\n" << (double)sum / mycont.size() << " " << mycont.size() << std::endl;
	for (const auto& elem : res)
		std::cout << elem << " ";
	return res;
}

template<typename _Input_Iter, typename _Des_Iter>
void modify_iter(const _Input_Iter iter_beg, const _Input_Iter iter_end, _Des_Iter des_iter)
{
	typename _Input_Iter::value_type sum = 0;
	int k = 0;

	for (auto it = iter_beg; it != iter_end; ++it)
		sum += *it;

	for (auto it = iter_beg; it != iter_end; ++it)
	{
		if (*it == 0)
			*(des_iter++) = ((double)sum / std::distance(iter_beg, iter_end));
		else
			*(des_iter++) = *it;
	}

}

template<typename _Container>
void modify_for_each(_Container& cont)
{
	typename _Container::value_type average = 0;

	for (auto& elem : cont)
		average += elem;
	average = (double)average / cont.size();

	std::for_each(cont.begin(), cont.end(), [&average](typename _Container::value_type& elem) 
		{
			if (elem == 0)
				elem = average;
		});	
}

void modify_transform(const std::string& input_fname, const std::string& output_fname)
{
	std::ifstream in_copy(input_fname);
	std::ifstream in_trans(input_fname);
	std::ofstream out(output_fname);

	if (!in_copy)
	{
		std::cerr << "File cannot be opened" << std::endl;
	}
	else
	{
		std::vector<int> cont(20);
		std::copy(std::istream_iterator<int>(in_copy), std::istream_iterator<int>(), cont.begin());
		int average = 0;

		for (auto& elem : cont)
			average += elem;
		average = (double)average / cont.size();
		std::transform(std::istream_iterator<int>(in_trans), std::istream_iterator<int>(), std::ostream_iterator<int>(out, " "), [&average](int elem)
			{
				if (elem == 0)
					return average;
				return elem;
			});
	}
}

template <typename _Container>
void outputResult_stream(const std::string& filename, const _Container& cont)
{
	std::ofstream out(filename);
	for (const auto& elem : cont)
		out << elem << " ";
}

template <typename _Container>
void outputResult_copy(const std::string& filename, const _Container& cont)
{
	std::ofstream out(filename);
	std::copy(cont.begin(), cont.end(), std::ostream_iterator<typename _Container::value_type>(out, " "));
}
